// Copyright (c) 2026 The Chromium Embedded Framework Authors. All rights
// reserved. Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.
//
// ---------------------------------------------------------------------------
//
// This file was generated by the CEF translator tool. If making changes by
// hand only do so within the body of existing method and function
// implementations. See the translator.README.txt file in the tools directory
// for more information.
//
// $hash=7b8ac61cb412f9c8806f3862c4e439b1ea52c31d$
//

#ifndef CEF_LIBCEF_DLL_CTOCPP_VIEWS_MENU_BUTTON_PRESSED_LOCK_CTOCPP_H_
#define CEF_LIBCEF_DLL_CTOCPP_VIEWS_MENU_BUTTON_PRESSED_LOCK_CTOCPP_H_
#pragma once

#if !defined(WRAPPING_CEF_SHARED)
#error This file can be included wrapper-side only
#endif

#include "include/capi/views/cef_menu_button_capi.h"
#include "include/capi/views/cef_menu_button_delegate_capi.h"
#include "include/views/cef_menu_button.h"
#include "include/views/cef_menu_button_delegate.h"
#include "libcef_dll/ctocpp/ctocpp_ref_counted.h"

// Wrap a C structure with a C++ class.
// This class may be instantiated and accessed wrapper-side only.
class CefMenuButtonPressedLockCToCpp
    : public CefCToCppRefCounted<CefMenuButtonPressedLockCToCpp,
                                 CefMenuButtonPressedLock,
                                 cef_menu_button_pressed_lock_t> {
 public:
  CefMenuButtonPressedLockCToCpp();
  virtual ~CefMenuButtonPressedLockCToCpp();

  // CefMenuButtonPressedLock methods.
};

constexpr auto CefMenuButtonPressedLockCToCpp_Wrap =
    CefMenuButtonPressedLockCToCpp::Wrap;
constexpr auto CefMenuButtonPressedLockCToCpp_Unwrap =
    CefMenuButtonPressedLockCToCpp::Unwrap;

#endif  // CEF_LIBCEF_DLL_CTOCPP_VIEWS_MENU_BUTTON_PRESSED_LOCK_CTOCPP_H_
