// Copyright (c) 2026 The Chromium Embedded Framework Authors. All rights
// reserved. Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.
//
// ---------------------------------------------------------------------------
//
// This file was generated by the CEF translator tool. If making changes by
// hand only do so within the body of existing method and function
// implementations. See the translator.README.txt file in the tools directory
// for more information.
//
// $hash=ec7d3697fbd701c98c6697bc58f583937ace2ae0$
//

#ifndef CEF_LIBCEF_DLL_CTOCPP_TEST_TRANSLATOR_TEST_SCOPED_LIBRARY_CHILD_CHILD_CTOCPP_H_
#define CEF_LIBCEF_DLL_CTOCPP_TEST_TRANSLATOR_TEST_SCOPED_LIBRARY_CHILD_CHILD_CTOCPP_H_
#pragma once

#if !defined(WRAPPING_CEF_SHARED)
#error This file can be included wrapper-side only
#endif

#include "include/capi/test/cef_translator_test_capi.h"
#include "include/test/cef_translator_test.h"
#include "libcef_dll/ctocpp/ctocpp_scoped.h"

// Wrap a C structure with a C++ class.
// This class may be instantiated and accessed wrapper-side only.
class CefTranslatorTestScopedLibraryChildChildCToCpp
    : public CefCToCppScoped<CefTranslatorTestScopedLibraryChildChildCToCpp,
                             CefTranslatorTestScopedLibraryChildChild,
                             cef_translator_test_scoped_library_child_child_t> {
 public:
  CefTranslatorTestScopedLibraryChildChildCToCpp();
  virtual ~CefTranslatorTestScopedLibraryChildChildCToCpp();

  // CefTranslatorTestScopedLibraryChildChild methods.
  int GetOtherOtherValue() override;
  void SetOtherOtherValue(int value) override;

  // CefTranslatorTestScopedLibraryChild methods.
  int GetOtherValue() override;
  void SetOtherValue(int value) override;

  // CefTranslatorTestScopedLibrary methods.
  int GetValue() override;
  void SetValue(int value) override;
};

constexpr auto CefTranslatorTestScopedLibraryChildChildCToCpp_Wrap =
    CefTranslatorTestScopedLibraryChildChildCToCpp::Wrap;
constexpr auto CefTranslatorTestScopedLibraryChildChildCToCpp_UnwrapOwn =
    CefTranslatorTestScopedLibraryChildChildCToCpp::UnwrapOwn;
constexpr auto CefTranslatorTestScopedLibraryChildChildCToCpp_UnwrapRaw =
    CefTranslatorTestScopedLibraryChildChildCToCpp::UnwrapRaw;

#endif  // CEF_LIBCEF_DLL_CTOCPP_TEST_TRANSLATOR_TEST_SCOPED_LIBRARY_CHILD_CHILD_CTOCPP_H_
